@@ -21,7 +21,7 @@ proc cvs_incvs {} {
 #
 #  returns: the current wd (ERROR) or the sandbox directory (OK)
 #
-proc cvs_sandbox_runcmd {cmd output_var} {
+proc cvs_sandbox_runcmd {command output_var} {
   global cvscfg
   global cwd
 
@@ -31,7 +31,7 @@ proc cvs_sandbox_runcmd {cmd output_var} {
   # needs to be seen by the server.  It can't cd to an absolute path.
   # In addition it's fussy about where you are when you do a checkout -d.
   # Best avoid that altogether.
-  gen_log:log T "ENTER ($cmd $output_var)"
+  gen_log:log T "ENTER ($command $output_var)"
   set pid [pid]
   
   if {! [file isdirectory $cvscfg(tmpdir)]} {
@@ -47,8 +47,8 @@ proc cvs_sandbox_runcmd {cmd output_var} {
   cd cvstmpdir.$pid
   gen_log:log F "CD [pwd]"
 
-  gen_log:log C "$cmd"
-  set ret [catch {eval "exec $cmd"} view_this]
+  gen_log:log C "$command"
+  set ret [catch {eval "exec $command"} view_this]
   gen_log:log T "RETURN $cvscfg(tmpdir)/cvstmpdir.$pid"
   return $cvscfg(tmpdir)/cvstmpdir.$pid
 }
@@ -110,6 +110,7 @@ proc cvs_workdir_status {} {
     # Status: or Sticky Tag:, putting each file's info (name, status, and tag)
     # into an array.
 
+    set datestatus_seen 0
     $cmd(cvs_status)\::destroy
     catch {unset cmd(cvs_status)}
     foreach logline $status_lines {
@@ -120,15 +121,11 @@ proc cvs_workdir_status {} {
         # Should be able to do these regsubs in one expression
         regsub {File: } [lindex $line 0] "" filename
         regsub {\s*$} $filename "" filename
-        #if {[string match "no file *" $filename]} {
-          #regsub {^no file } $filename "" filename
-        #}
         regsub {Status: } [lindex $line 1] "" status
         set Filelist($filename:status) $status
         # Don't set editors to null because we'll use its presence
         # or absence to see if we need to re-read the repository when
         # we ask to map the editors column
-        #set Filelist($filename:editors) ""
       } elseif {[string match "*Working revision:*" $logline]} {
         regsub -all {\t+} $logline "\t" logline
         set line [split [string trim $logline] "\t"]
@@ -141,7 +138,11 @@ proc cvs_workdir_status {} {
         if {$date == "" } {
          if {! ([string match "New *" $date ] || [string match "Result *" $date])} {
            catch {set date [clock format [file mtime $filename] -format $cvscfg(dateformat)]}
-           gen_log:log E "No date supplied by remote CVS server. Using \[file mtime $filename\]"
+           if {! $datestatus_seen} {
+             # We only need to see this message once per directory
+             set datestatus_seen 1
+             gen_log:log E "No date supplied by remote CVS server. Using \[file mtime\]"
+           }
          }
         } else {
           # CVS outputs time strings tcl can't handle, such as
@@ -272,8 +273,8 @@ proc cvs_remove {args} {
     return
   }
 
-  set cmd [exec::new "$cvs remove $filelist"]
-  auto_setup_dir $cmd
+  set cmd(cvscmd) [exec::new "$cvs remove $filelist"]
+  auto_setup_dir $cmd(cvscmd)
 
   gen_log:log T "LEAVE"
 }
@@ -446,8 +447,8 @@ proc cvs_add {binflag args} {
   if {$filelist == ""} {
     append filelist [glob -nocomplain $cvscfg(aster) .??*]
   }
-  set cmd [exec::new "$cvs add $binflag $filelist"]
-  auto_setup_dir $cmd
+  set cmd(cvscmd) [exec::new "$cvs add $binflag $filelist"]
+  auto_setup_dir $cmd(cvscmd)
 
   gen_log:log T "LEAVE"
 }
@@ -671,6 +672,27 @@ proc cvs_log {args} {
   gen_log:log T "LEAVE"
 }
 
+# called from the branch browser
+proc cvs_log_rev {rev file} {
+  global cvs
+
+  gen_log:log T "ENTER ($rev $file)"
+
+  set title "CVS log"
+  set commandline "$cvs log -N"
+  if {$rev ne ""} {
+    append commandline " -r$rev"
+    append title " -r$rev"
+  }
+  append commandline " $file"
+  append title " $file"
+
+  set logcmd [viewer::new "$title"]
+  $logcmd\::do "$commandline" 0 hilight_rcslog
+
+  gen_log:log T "LEAVE"
+}
+
 proc cvs_annotate {revision args} {
 #
 # This looks at a log from the repository.
@@ -804,6 +826,7 @@ proc cvs_tag {tagname force b_or_t update args} {
 #
   global cvs
   global cvscfg
+  global cvsglb
   global incvs
 
   gen_log:log T "ENTER ($tagname $force $b_or_t $update $args)"
@@ -832,7 +855,6 @@ proc cvs_tag {tagname force b_or_t update args} {
   if {$b_or_t == "branch" && $force == "yes"} {
     set too_new 0
     # As of 1.11.2, -F won't move branch tags without the -B option
-    set cvsglb(cvs_version) [cvs_version_number]
     set versionsplit [split $cvsglb(cvs_version) {.}]
     set major [lindex $versionsplit 1]
     set minor [lindex $versionsplit 2]
@@ -977,6 +999,11 @@ proc cvs_merge {parent from since frombranch args} {
   global cvsglb
 
   gen_log:log T "ENTER (\"$from\" \"$since\" \"$frombranch\" \"$args\")"
+  gen_log:log D "mergetrunkname $cvscfg(mergetrunkname)"
+  set realfrom "$frombranch"
+  if {$frombranch eq $cvscfg(mergetrunkname)} {
+    set realfrom "HEAD"
+  }
 
   set filelist [join $args]
 
@@ -998,9 +1025,9 @@ proc cvs_merge {parent from since frombranch args} {
 
   # Do the update here, and defer the tagging until later
   if {$since == {}} {
-    set commandline "$cvs update -d -j$from $filelist"
+    set commandline "$cvs update -d -j$realfrom $filelist"
   } else {
-    set commandline "$cvs update -d -j$since -j$from $filelist"
+    set commandline "$cvs update -d -j$since -j$realfrom $filelist"
   }
   set v [viewer::new "CVS Join"]
   $v\::do "$commandline" 1 status_colortags
@@ -1029,6 +1056,10 @@ proc cvs_merge_tag_seq {from frombranch totag fromtag args} {
   foreach f $args {
     append filelist "\"$f\" "
   }
+  set realfrom "$frombranch"
+  if {$frombranch eq $cvscfg(mergetrunkname)} {
+    set realfrom "HEAD"
+  }
 
   # It's muy importante to make sure everything is OK at this point
   set commandline "$cvs -n -q update $filelist"
@@ -1052,7 +1083,7 @@ proc cvs_merge_tag_seq {from frombranch totag fromtag args} {
   # Tag if desired
   if {$cvscfg(auto_tag) && $totag != ""} {
     # First, the "from" file that's not in this branch (needs -r)
-    set commandline "$cvs tag -F -r$from $totag $filelist"
+    set commandline "$cvs tag -F -r$realfrom $totag $filelist"
     $v\::log "$commandline\n"
     $v\::do "$commandline" 1
     $v\::wait
@@ -1344,22 +1375,40 @@ proc cvs_patch { cvsroot module difffmt revtagA dateA revtagB dateB outmode outf
   return
 }
 
-proc cvs_version_number {} {
+proc cvs_version {} {
 #
 # This finds the current CVS version number.
 #
   global cvs
   global cvscfg
+  global cvsglb
 
   gen_log:log T "ENTER"
+  set cvsglb(cvs_version) ""
+
   set commandline "$cvs -v"
-  set e [exec::new "$commandline" {} 0 parse_version]
-  set number [$e\::output]
-  regsub -all {\s*} $number {} number
+  gen_log:log C "$commandline"
+  set ret [catch {eval "exec $commandline"} output]
+  if {$ret} {
+    cvsfail $output
+    return
+  }
+  foreach infoline [split $output "\n"] {
+    if {[string match "Concurrent*" $infoline]} {
+      set lr [split $infoline]
+      set species [lindex $lr 3]
+      regsub -all {[()]} $species {} species
+      set version [lindex $lr 4]
+      gen_log:log D "species $species   version $version"
+    }
+  }
+  gen_log:log D "Split: $species $version"
+  regsub -all {\s*} $version {} version
+  gen_log:log D "De-whitespaced: $species $version"
+  set cvsglb(cvs_type) $species
+  set cvsglb(cvs_version) $version
   
-  $e\::destroy
-  gen_log:log T "LEAVE ($number)"
-  return $number
+  gen_log:log T "LEAVE"
 }
 
 proc cvs_merge_conflict {args} {
@@ -1678,10 +1727,8 @@ proc cvs_ascii { args } {
 
   gen_log:log D "Changing sticky flag"
   gen_log:log D "$cvs admin -kkv $filelist"
-  set cmd [exec::new "$cvs admin -kkv $filelist"]
-  # gen_log:log D "Updating file list"
-  # set cmd [exec::new "$cvs update $filelist"]
-  auto_setup_dir $cmd
+  set cmd(cvscmd) [exec::new "$cvs admin -kkv $filelist"]
+  auto_setup_dir $cmd(cvscmd)
 
   gen_log:log T "LEAVE"
 }
@@ -1702,10 +1749,8 @@ proc cvs_binary { args } {
 
   gen_log:log D "Changing sticky flag"
   gen_log:log D "$cvs admin -kb $filelist"
-  set cmd [exec::new "$cvs admin -kb $filelist"]
-  # gen_log:log D "Updating file list"
-  # set cmd [exec::new "$cvs update $filelist"]
-  auto_setup_dir $cmd
+  set cmd(cvscmd) [exec::new "$cvs admin -kb $filelist"]
+  auto_setup_dir $cmd(cvscmd)
 
   gen_log:log T "LEAVE"
 }
@@ -1715,6 +1760,7 @@ proc cvs_binary { args } {
 proc cvs_revert {args} {
   global incvs
   global cvscfg
+  global cvsglb
   global cvs
 
   gen_log:log T "ENTER ($args)"
@@ -1736,18 +1782,17 @@ proc cvs_revert {args} {
 
   gen_log:log D "Reverting $filelist"
   # update -C option appeared in 1.11
-  set cvsglb(cvs_version) [cvs_version_number]
   set versionsplit [split $cvsglb(cvs_version) {.}]
   set major [lindex $versionsplit 1]
   if {$major < 11} {
     gen_log:log F "DELETE $filelist"
     file delete $filelist
-    set cmd [exec::new "$cvs update $filelist"]
+    set cmd(cvscmd) [exec::new "$cvs update $filelist"]
   } else {
-    set cmd [exec::new "$cvs update -C $filelist"]
+    set cmd(cvscmd) [exec::new "$cvs update -C $filelist"]
   }
   
-  auto_setup_dir $cmd
+  auto_setup_dir $cmd(cvscmd)
 
   gen_log:log T "LEAVE"
 }
@@ -1763,6 +1808,9 @@ proc read_cvs_dir {dirname} {
   global current_tagname
 
   gen_log:log T "ENTER ($dirname)"
+  if {$cvsglb(cvs_version) == ""} {
+    cvs_version
+  }
   set current_tagname "trunk"
   if {[file isdirectory $dirname]} {
     if {[file isfile [file join $dirname Repository]]} {
@@ -1799,17 +1847,6 @@ proc read_cvs_dir {dirname} {
     return 0
   }
   set cvsglb(root) $cvscfg(cvsroot)
-  # This confirms whether we can really use cvs but do we
-  # want to waste time doing it?
-  #gen_log:log D "cvsglb(root) $cvsglb(root)"
-  #gen_log:log D "cvscfg(cvsroot) $cvscfg(cvsroot)"
-  #set command "$cvs version"
-  #gen_log:log C "$command"
-  #set ret [catch {eval "exec $command"} output]
-  #if {$ret} {
-    #cvsfail $output
-    #return 0
-  #}
 
   gen_log:log T "LEAVE (1)"
   return 1
@@ -1877,8 +1914,8 @@ proc cvs_lock {do files} {
     lock { set commandline "cvs admin -l $files"}
     unlock { set commandline "cvs admin -u $files"}
   }
-  set cmd [::exec::new "$commandline"]
-  auto_setup_dir $cmd
+  set lock_cmd [::exec::new "$commandline"]
+  auto_setup_dir $lock_cmd
 }
 
 # Sends directory "." to the directory-merge tool
@@ -1988,6 +2025,7 @@ namespace eval ::cvs_branchlog {
       variable revlines
       variable revstate
       variable revcomment
+      variable revmergefrom
       variable tags
       variable revbranches
       variable branchrevs
@@ -2047,6 +2085,7 @@ namespace eval ::cvs_branchlog {
         variable revlines
         variable revstate
         variable revcomment
+        variable revmergefrom
         variable revtags
         variable revbtags
         variable revbranches
@@ -2061,6 +2100,7 @@ namespace eval ::cvs_branchlog {
         catch { unset revlines }
         catch { unset revstate }
         catch { unset revcomment }
+        catch { unset revmergefrom }
         catch { unset revtags }
         catch { unset revbtags }
         catch { unset revbranches }
@@ -2102,6 +2142,7 @@ namespace eval ::cvs_branchlog {
         variable revlines
         variable revstate
         variable revcomment
+        variable revmergefrom
         variable revtags
         variable revbtags
         variable revbranches
@@ -2227,34 +2268,33 @@ namespace eval ::cvs_branchlog {
             }
             {D} {
               # Look for a date line.  This also has the name of the author.
-              set parts [split $logline]
-	      if {[lindex $parts 4] == "author:"} {
-                foreach [list \
-                    revwho($rnum) revdate($rnum) revtime($rnum) \
-                    revlines($rnum) revstate($rnum) \
-                  ] \
-                  [list \
-                    [string trimright [lindex $parts 5] {;}] \
-                    [lindex $parts 1] \
-                    [string trimright [lindex $parts 2] {;}] \
-                    [lrange $parts 11 end] \
-                    [string trimright [lindex $parts 8] {;}] \
-                  ] \
-                  { break }
-	      } else {
-                foreach [list \
-                    revwho($rnum) revdate($rnum) revtime($rnum) \
-                    revlines($rnum) revstate($rnum) \
-                  ] \
-                  [list \
-                    [string trimright [lindex $parts 6] {;}] \
-                    [lindex $parts 1] \
-                    [string trimright [lindex $parts 2] {;}] \
-                    [lrange $parts 11 end] \
-                    [string trimright [lindex $parts 8] {;}] \
-                  ] \
-                  { break }
-	      }
+              set parts [split $logline ";"]
+              foreach p $parts {
+                set eqn [split $p ":"];
+                set eqname [string trim [lindex $eqn 0]]
+                set eqval  [string trim [join [lrange $eqn 1 end] ":"]]
+                switch -exact -- $eqname {
+                  {date} {
+                    set revdate($rnum) [lindex $eqval 0]
+                    set revtime($rnum) [lindex $eqval 1]
+                    gen_log:log D "date $revdate($rnum)"
+                    gen_log:log D "time $revtime($rnum)"
+                  }
+                  {author} {
+                    set revwho($rnum) $eqval
+                  }
+                  {lines} {
+                    set revlines($rnum) $eqval
+                  }
+                  {state} {
+                    set revstate($rnum) $eqval
+                  }
+                  {mergepoint} {
+                    set revmergefrom($rnum) $eqval
+                    gen_log:log D "mergefrom $revmergefrom($rnum)"
+                  }
+                }
+              }
               set logstate {L}
             }
             {L} {
@@ -2288,7 +2328,6 @@ namespace eval ::cvs_branchlog {
       proc cvs_sort_it_all_out {} {
         global cvscfg
         global module_dir
-        #global current_tagname
         variable filename
         variable sys
         variable lc
@@ -2299,6 +2338,7 @@ namespace eval ::cvs_branchlog {
         variable revlines
         variable revstate
         variable revcomment
+        variable revmergefrom
         variable revtags
         variable revbtags
         variable revbranches
@@ -2371,14 +2411,12 @@ namespace eval ::cvs_branchlog {
                   # there is no log and no revisions to show.
                   # FIXME: what if this is a resurrection?
                   lappend branchrevs(trunk) {current}
-                  #set revbranches(current) {}
                 } elseif {[info exists rootbranch($tag)] && \
                     $rootbranch($tag) == $rnum} {
                   # The sticky tag specifies a branch and the branch's
                   # root is the same as the source revision. Place the
                   # you-are-here box at the start of the branch.
                   lappend branchrevs($revbranch($tag)) {current}
-                  #set revbranches(current) {}
                 } else {
                   if {[catch {info exists $branchrevs($root)}] == 0} {
                     if {$rnum == [lindex $branchrevs($root) 0]} {
@@ -2387,7 +2425,6 @@ namespace eval ::cvs_branchlog {
                       # branch.
                       set branchrevs($root) [linsert $branchrevs($root) 0\
                         {current}]
-                      #set revbranches(current) {}
                     } else {
                       # Otherwise we will place it as a branch off the
                       # revision.
