@@ -1,9 +1,7 @@
 #
-# TCL Library for TkCVS
+# Tcl Library for TkCVS
 #
 
-#
-# $Id: modules.tcl,v 1.27 2002/11/10 05:56:14 dorothyr Exp $
 #
 # Procedures to parse the CVS modules file and store whatever is
 # read into various associative arrays, sorted, and unsorted lists.
@@ -21,54 +19,6 @@
 # filenames
 #   For each module, the list of files that it contains.
 
-proc parse_modules {modules_file} {
-  global cvs
-  global modval
-  global modtitle
-  global cvsglb
-  global cvscfg
-
-  gen_log:log T "ENTER"
-
-  # Clear the arrays
-  catch {unset modval}
-  catch {unset modtitle}
-
-  # Unescape newlines, compress repeated whitespace, and remove blank lines
-  regsub -all {(\\\n|[ \t])+} $modules_file " " modules_file
-  regsub -all {\n\s*\n+} $modules_file "\n" modules_file
-
-  foreach line [split $modules_file "\n"] {
-    if {[string index $line 0] == {#}} {
-#     gen_log:log D "Comment: $line"
-      if {[string index $line 1] == {D} || [string index $line 1] == {M}} {
-        set text [split $line]
-        set dname [lindex $text 1]
-        set modtitle($dname) [lrange $text 2 end]
-#       gen_log:log D "Directory: {$dname} {$modtitle($dname)}"
-      }
-    } else {
-#     gen_log:log D "Data: $line"
-      set text [split $line]
-      set modname [lindex $text 0]
-      set modstring [string trim [join [lrange $text 1 end]]]
-      # A "#D ..." or "#M ..." entry _always_ overrides this default
-      if {! [info exists modtitle($modname)]} {
-        set modtitle($modname) $modstring
-      }
-      # Remove flags except for -a.  Luckily alias modules can't have
-      # any other options.
-#     gen_log:log D "{$modname} {$modstring}"
-      regsub -- {^((-l\s*)|(-[ioestud]\s+((\\\s)|\S)+\s*))+} \
-        $modstring {} modstring
-#     gen_log:log D "{$modname} {$modstring}"
-      set modval($modname) $modstring
-    }
-  }
-
-  gen_log:log T "LEAVE"
-}
-
 proc gather_mod_index {} {
 #
 # Creates a new global list called modlist for the report printouts
@@ -80,7 +30,7 @@ proc gather_mod_index {} {
   global modlist
   global modlist_sorted
 
-  gen_log:log T "ENTER ()"
+  #gen_log:log T "ENTER ()"
   set modlist {}
   set dlist {}
   if {! [info exists modtitle]} {
@@ -116,21 +66,21 @@ proc gather_mod_index {} {
   set modlist_sorted [lsort $modlist]
   if {$cvscfg(logging) && [regexp -nocase {d} $cvscfg(log_classes)]} {
     foreach idx $modlist_sorted {
-      gen_log:log D "$idx"
+      #gen_log:log D "$idx"
       set dname [lindex $idx 0]
       if {[info exists dparent($dname)]} {
-        gen_log:log D "   PARENT: $dparent($dname)"
+        #gen_log:log D "   PARENT: $dparent($dname)"
       }
       if {[info exists dcontents($dname)]} {
-        gen_log:log D "   CHILDREN: $dcontents($dname)"
+        #gen_log:log D "   CHILDREN: $dcontents($dname)"
       }
       set desc [find_subdirs $dname 0]
       if {$desc != ""} {
-        gen_log:log D "   SUBDIRS: $desc"
+        #gen_log:log D "   SUBDIRS: $desc"
       }
     }
   }
-  gen_log:log T "LEAVE"
+  #gen_log:log T "LEAVE"
 }
 
 proc find_filenames {mcode} {
@@ -168,12 +118,12 @@ proc find_filenames {mcode} {
    
   set view_lines [split $view_this "\n"]
   foreach line $view_lines {
-    gen_log:log D "$line"
+    #gen_log:log D "$line"
     if {[string match "File *" $line]} {
       set lst [split $line]
       set cut [expr {[llength $lst] - 6}]
       set dname [join [lrange $lst 1 $cut]]
-      gen_log:log D "$dname"
+      #gen_log:log D "$dname"
       lappend filenames($mcode) $dname
     }
   }
