@@ -1,3 +1,41 @@
+proc svn_version {} {
+  global cvsglb
+
+  gen_log:log T "ENTER"
+
+  # We don't need the version.  We just need to know if mergeinfo is
+  # going to work in this server/client combination.  But let's not forget
+  # how to get the version.
+
+  #set cvsglb(svn_version) ""
+  #set commandline "svn --version"
+  #gen_log:log C "$commandline"
+  #set ret [catch {eval "exec $commandline"} output]
+  #if {$ret} {
+  #  cvsfail $output
+  #  return
+  #}
+  #foreach infoline [split $output "\n"] {
+  #  if {[string match "svn,*" $infoline]} {
+  #    set lr [split $infoline]
+  #    set version [lindex $lr 2]
+  #    gen_log:log D "version $version"
+  #  }
+  #}
+  #set cvsglb(svn_version) $version
+
+  set commandline "svn log -g"
+  set ret [catch {eval "exec $commandline"} output]
+  if {$ret == 0} {
+    set cvsglb(svn_mergeinfo_works) 1
+    gen_log:log D "svn mergeinfo works"
+  } else {
+    set cvsglb(svn_mergeinfo_works) 0
+    gen_log:log D "svn mergeinfo doesn't work"
+  }
+  gen_log:log T "LEAVE"
+}
+
 # Find SVN URL
 proc read_svn_dir {dirname} {
   global cvscfg
@@ -7,6 +45,9 @@ proc read_svn_dir {dirname} {
   global cmd
 
   gen_log:log T "ENTER ($dirname)"
+  # Whether mergeinfo works depends on the server as well as the local svn program,
+  # so it may work for us in one repository but not another
+  svn_version
   # svn info gets the URL
   # Have to do eval exec because we need the error output
   set command "svn info"
@@ -18,8 +59,8 @@ proc read_svn_dir {dirname} {
   }
   foreach infoline [split $output "\n"] {
     if {[string match "URL*" $infoline]} {
-      gen_log:log D "$infoline"
       set cvscfg(url) [lrange $infoline 1 end]
+      gen_log:log D "$cvscfg(url)"
     }
   }
 
@@ -32,7 +73,7 @@ proc read_svn_dir {dirname} {
   }
 
   set root ""
-  foreach s {trunk branches tags} {
+  foreach s [list $cvscfg(svn_trunkdir) $cvscfg(svn_branchdir) $cvscfg(svn_tagdir)] {
     if {[regexp "/$s/" $cvscfg(url)] || [regexp "/$s" $cvscfg(url)]} {
       set spl [split $cvscfg(url) "/"]
       set root ""
@@ -43,22 +84,23 @@ proc read_svn_dir {dirname} {
         set word [lindex $spl $j]
         switch -- $state {
           P {
-            switch -- $word {
-              "trunk" {
-                set type $word
+            if {$word eq $cvscfg(svn_trunkdir)} {
+                gen_log:log D "Matched $word for trunk"
+                set type "trunk"
                 set current_tagname $word
                 set state E
-              } 
-              "branches" {
-                set type $word
+            } elseif { $word eq $cvscfg(svn_branchdir)} {
+                gen_log:log D "Matched $word for branches"
+                set type "branches"
                 set state W
-              }
-              "tags" {
-                set type $word
+            } elseif { $word eq $cvscfg(svn_tagdir)} {
+                gen_log:log D "Matched $word for tags"
+                set type "tags"
                 set state W
-              }
-              default { append root "$word/" }
-             }
+            } else {
+                append root "$word/"
+                #gen_log:log D "No match for $word"
+            }
           }
           W {
             set current_tagname $word
@@ -71,6 +113,7 @@ proc read_svn_dir {dirname} {
         }
       }
       set cvscfg(svnroot) [string trimright $root "/"]
+      set cvsglb(root) $cvscfg(svnroot)
       gen_log:log D "SVN URL: $cvscfg(url)"
       gen_log:log D "svnroot: $cvscfg(svnroot)"
       set cvsglb(relpath) [join $relp {/}]
@@ -80,14 +123,19 @@ proc read_svn_dir {dirname} {
     }
   }
   if {$root == ""} {
-    gen_log:log D "Nonconforming repository"
+    gen_log:log F "Nonconforming repository"
+    puts "No conforming $cvscfg(svn_trunkdir)/$cvscfg(svn_branchdir)/$cvscfg(svn_tagdir) structure detected in the repository"
+    puts " I won't be able to detect any branches or tags."
     gen_log:log D "SVN URL: $cvscfg(url)"
     set cvscfg(svnroot) $cvscfg(url)
+    set cvsglb(root) $cvscfg(svnroot)
     gen_log:log D "svnroot: $cvscfg(svnroot)"
     set cvsglb(relpath) ""
+    set cvsglb(svnconform) 0
     gen_log:log T "LEAVE (-1)"
     return -1
   }
+  set cvsglb(svnconform) 1
   gen_log:log T "LEAVE (0)"
   return 1
 }
@@ -187,8 +235,8 @@ proc svn_add {args} {
   if {$filelist == ""} {
     append filelist [glob -nocomplain $cvscfg(aster) .??*]
   }
-  set cmd [exec::new "svn add $filelist"]
-  auto_setup_dir $cmd
+  set addcmd [exec::new "svn add $filelist"]
+  auto_setup_dir $addcmd
 
   gen_log:log T "LEAVE"
 }
@@ -284,10 +332,10 @@ proc svn_opt_update {} {
        set command "svn update"
      }
     "Trunk" {
-       set command "svn switch $cvscfg(svnroot)/trunk"
+       set command "svn switch $cvscfg(svnroot)/$cvscfg(svn_trunkdir)"
      }
     "Branch" {
-       set command "svn switch $cvscfg(svnroot)/branches/$cvsglb(branchname)"
+       set command "svn switch $cvscfg(svnroot)/$cvscfg(svn_branchdir)/$cvsglb(branchname)"
      }
     "Revision" {
        # Let them get away with saying r3 instead of 3
@@ -717,11 +765,23 @@ proc parse_svnmodules {tf svnroot} {
 # called from workdir Reports menu
 proc svn_log {args} {
   global cvscfg
+  global cvsglb
+
   gen_log:log T "ENTER ($args)"
 
+  set svncommand "svn log "
+  # svn -g (mergeinfo) appeared in 1.5.  It depends on the server
+  # as well as the client, so we can't go by version number.  we
+  # just have to see if it works.
+  # Do we want to do -g for all detail levels?  Probably not for summary.
+  if {$cvsglb(svn_mergeinfo_works)} {
+    if {$cvscfg(ldetail) ne "summary"} {
+      append svncommand "-g "
+    }
+  }
   set filelist [join $args]
   foreach file $filelist {
-    set command "svn log "
+    set command $svncommand
     if {$cvscfg(ldetail) == "latest"} {
       append command "-r COMMITTED "
     }
@@ -736,6 +796,29 @@ proc svn_log {args} {
   gen_log:log T "LEAVE"
 }
 
+# called from branch browser
+proc svn_log_rev {filepath} {
+  global cvscfg
+  global cvsglb
+
+  gen_log:log T "ENTER ($filepath)"
+
+  set svncommand "svn log "
+  # svn -g (mergeinfo) appeared in 1.5.  It depends on the server
+  # as well as the client, so we can't go by version number.  we
+  # just have to see if it works.
+  if {$cvsglb(svn_mergeinfo_works)} {
+    append svncommand "-g "
+  }
+  if {[regexp {/} $filepath]} {
+    append svncommand "--stop-on-copy "
+  }
+  append svncommand "\"$filepath\""
+  set logcmd [viewer::new "SVN log $filepath"]
+  $logcmd\::do "$svncommand"
+  gen_log:log T "LEAVE"
+}
+
 proc svn_merge_conflict {args} {
   global cvscfg
 
@@ -766,8 +849,8 @@ proc svn_merge_conflict {args} {
       cvsfail "$file does not appear to have a conflict." .workdir
       continue
     }
-  # FIXME: we don't want to tie up the whole UI with tkdiff, but
-  # if we don't wait, we have no way to know if we can mark resolved
+    # FIXME: we don't want to tie up the whole UI with tkdiff, but
+    # if we don't wait, we have no way to know if we can mark resolved
     # Invoke tkdiff with the proper option for a conflict file
     # and have it write to the original file
     set command "$cvscfg(tkdiff) -conflict -o \"$file\" \"$file\""
@@ -859,8 +942,8 @@ proc svn_tag {tagname b_or_t update args} {
   set filelist [join $args]
   gen_log:log D "relpath: $cvsglb(relpath)  filelist \"$filelist\""
 
-  if {$b_or_t == "tag"} {set pathelem "tags"}
-  if {$b_or_t == "branch"} {set pathelem "branches"}
+  if {$b_or_t == "tag" || $b_or_t == "tags"} {set pathelem "$cvscfg(svn_tagdir)"}
+  if {$b_or_t == "branch"} {set pathelem "$cvscfg(svn_branchdir)"}
 
   set comment "${b_or_t}_copy_by_TkSVN"
   set v [viewer::new "SVN Copy $tagname"]
@@ -914,13 +997,13 @@ proc svn_rcopy {from_path b_or_t newtag} {
   incr idx ;# advance past /
   set from_path_remainder [string range $from_path $idx end]
   set pathelements [file split $from_path_remainder]
-  if {$from_path_remainder == "trunk"} {
+  if {$from_path_remainder == "$cvscfg(svn_trunkdir)"} {
     set need_list 1
   } elseif {[llength $pathelements] == 2} {
     set from_type [lindex $pathelements 0]
     switch -- $from_type {
-      "tags" -
-      "branches" {
+      "$cvscfg(svn_tagdir)" -
+      "$cvscfg(svn_branchdir)" {
         set need_list 1
       }
     }
@@ -1029,7 +1112,7 @@ proc svn_merge {parent frompath since currentpath frombranch args} {
   }
 
   # Do the update here, and defer the tagging until later
-  set commandline "svn merge \"$currentpath\" \"$frompath\""
+  set commandline "svn merge --accept postpone \"$currentpath\" \"$frompath\""
   set v [viewer::new "SVN Merge"]
   $v\::do "$commandline" 1 status_colortags
   $v\::wait
@@ -1082,9 +1165,9 @@ proc svn_merge_tag_seq {from frombranch totag fromtag args} {
   # Tag if desired (no means not a branch
   if {$cvscfg(auto_tag) && $fromtag != ""} {
     if {$frombranch == "trunk"} {
-      set from_path "$cvscfg(svnroot)/trunk/$cvsglb(relpath)"
+      set from_path "$cvscfg(svnroot)/$cvscfg(svn_trunkdir)/$cvsglb(relpath)"
     } else {
-      set from_path "$cvscfg(svnroot)/branches/$frombranch/$cvsglb(relpath)"
+      set from_path "$cvscfg(svnroot)/$cvscfg(svn_branchdir)/$frombranch/$cvsglb(relpath)"
     }
     # tag the current (mergedto) branch
     svn_tag $fromtag tags false $args ;# opens its own viewer
@@ -1157,10 +1240,20 @@ proc svn_filecat {root path title} {
 
 # SVN log.  Called from module browser
 proc svn_filelog {root path title} {
+  global cvsglb
+
   gen_log:log T "ENTER ($root $path $title)"
 
+  set command "svn log "
+  # svn -g (mergeinfo) appeared in 1.5.  It depends on the server
+  # as well as the client, so we can't go by version number.  we
+  # just have to see if it works.
+  if {$cvsglb(svn_mergeinfo_works)} {
+    append command "-g "
+  }
+
   set url [safe_url $root/$path]
-  set command "svn log \"$url\""
+  append command "\"$url\""
   set wintitle "SVN Log"
 
   set v [viewer::new "$wintitle $url"]
@@ -1270,6 +1363,8 @@ namespace eval ::svn_branchlog {
       variable revbranches
       variable branchrevs
       variable logstate
+      variable show_tags
+      variable show_merges
 
       gen_log:log T "ENTER [namespace current]"
       if {$directory_merge} {
@@ -1324,6 +1419,7 @@ namespace eval ::svn_branchlog {
         variable revname
         variable revtags
         variable revbtags
+        variable revmergefrom
         variable branchrevs
         variable allrevs
         variable revbranches
@@ -1331,6 +1427,7 @@ namespace eval ::svn_branchlog {
         variable relpath
         variable filename
         variable show_tags
+        variable show_merges
 
         gen_log:log T "ENTER"
         catch { $lc.canvas delete all }
@@ -1340,6 +1437,7 @@ namespace eval ::svn_branchlog {
         catch { unset revcomment }
         catch { unset revtags }
         catch { unset revbtags }
+        catch { unset revmergefrom }
         catch { unset branchrevs }
         catch { unset revbranches }
         catch { unset revkind }
@@ -1355,47 +1453,49 @@ namespace eval ::svn_branchlog {
         set path "$cvscfg(url)/$safe_filename"
         $ln\::ConfigureButtons $filename
 
+        set show_merges [set $ln\::opt(show_merges)]
+        set show_tags [set $ln\::opt(show_tags)]
+
         # Find out where to put the working revision icon (if anywhere)
-        set command "svn log -q --stop-on-copy \"$filename\""
-        set cmd_log [exec::new $command]
-        set log_output [$cmd_log\::output]
-        $cmd_log\::destroy
-        set loglines [split $log_output "\n"]
-        set svnstat [lindex $loglines 1]
-        set revnum_current [lindex $svnstat 0]
-        gen_log:log D "revnum_current $revnum_current"
+        set revnum_current [set $ln\::revnum_current]
+        set revnum_current r$revnum_current
 
         if { $relpath == {} } {
-          set path "$cvscfg(svnroot)/trunk/$safe_filename"
+          set path "$cvscfg(svnroot)/$cvscfg(svn_trunkdir)/$safe_filename"
         } else {
-          set path "$cvscfg(svnroot)/trunk/$relpath/$safe_filename"
+          set path "$cvscfg(svnroot)/$cvscfg(svn_trunkdir)/$relpath/$safe_filename"
         }
-        if {[read_svn_dir .] == -1} {
+        if {! $cvsglb(svnconform)} {
           set path "$cvscfg(svnroot)/$safe_filename"
-          if {! [info exists cvscfg(svnconform_seen)]} {
-            set msg "Your repository does not seem to be arranged in trunk, branch, and root directories.  The Branch Browser can't detect branches and tags."
-            cvsok "$msg" $lc
-            set cvscfg(svnconform_seen) 1
+        }
+        # We need to go to the repository to find the highest revision.  Doing
+        # info on local files may not have it.  Let's start with what we've got
+        # though in case it fails.
+        set highest_revision [string trimleft $revnum_current "r"]
+        set command "svn info $path"
+        gen_log:log C "$command"
+        set ret [catch {eval "exec $command"} output]
+        if {$ret} {
+          cvsfail $output
+        }
+        foreach infoline [split $output "\n"] {
+          if {[string match "Revision*" $infoline]} {
+            set highest_revision [lrange $infoline 1 end]
+            gen_log:log D "$highest_revision"
           }
         }
+
         # The trunk
         set branchrevs(trunk) {}
-        # if the file was added on a branch, this will error out.
-        # Come to think of it, there's nothing especially privileged
-        # about the trunk except that one branch must not stop-on-copy
-        set command "svn log $path"
+        # There's nothing especially privileged about the trunk except that one
+        # branch must not stop-on-copy.  Maybe the file was added on a branch,
+        # or maybe it isn't on the trunk anymore but it once was.  We'll have
+        # to use a range from r1 that case, to find it
+        set range "${highest_revision}:1"
+        set command "svn log -r $range $path"
         set cmd_log [exec::new $command {} 0 {} 1]
         set log_output [$cmd_log\::output]
         $cmd_log\::destroy
-        if {$log_output == ""} {
-          # Maybe the file isn't on the trunk anymore but it once was.
-          set j [string trimleft $revnum_current "r"]
-          set range "${j}:1"
-          set command "svn log -r $range $path"
-          set cmd_log [exec::new $command {} 0 {} 1]
-          set log_output [$cmd_log\::output]
-          $cmd_log\::destroy
-        }
         set trunk_lines [split $log_output "\n"]
         set rr [parse_svnlog $trunk_lines trunk]
         # See if the current revision is on the trunk
@@ -1404,6 +1504,28 @@ namespace eval ::svn_branchlog {
         set tip [lindex $brevs 0]
         set revpath($tip) $path
         set revkind($tip) "revision"
+        foreach r $branchrevs(trunk) {
+          if {$cvsglb(svn_mergeinfo_works) && $show_merges} {
+            set propcmd "svn propget svn:mergeinfo -$r $path"
+            set cmd_prop [exec::new $propcmd {} 0 {} 1]
+            set prop_output [$cmd_prop\::output]
+            $cmd_prop\::destroy
+            # It will probably be empty the majority of the time
+            # so don't waste time on empty output
+            if {$prop_output eq ""} {continue}
+            set splits [split $prop_output "\n"]
+            # FIXME: what if more than one merge comes into the same rev?
+            foreach line $splits {
+              if {$line eq ""} {continue}
+              set spl [split $line ":"]
+              set fromrevs [lindex $spl 1]
+              gen_log:log D "  to $r  fromrevs $fromrevs"
+              regsub {^.*-} $fromrevs {} lastfromrev 
+              set revmergefrom($r) "r$lastfromrev"
+              gen_log:log D "  revmergefrom($r) $revmergefrom($r)"
+            }
+          }
+        }
         set brevs [lreplace $brevs 0 0]
         if {$tip == $revnum_current} {
           # If current is at end of trunk do this.
@@ -1419,7 +1541,6 @@ namespace eval ::svn_branchlog {
           set revkind($r) "revision"
           set revpath($r) $path
         }
-        #set branchrevs($rr) [lrange $branchrevs(trunk) 0 end-1]
         set branchrevs($rr) $branchrevs(trunk)
         set revkind($rr) "root"
         set revname($rr) "trunk"
@@ -1427,35 +1548,25 @@ namespace eval ::svn_branchlog {
         set revpath($rr) $path
 
         # Branches
-        set command "svn list $cvscfg(svnroot)/branches"
+        # Get a list of the branches from the repository
+        set command "svn list $cvscfg(svnroot)/$cvscfg(svn_branchdir)"
         set cmd_log [exec::new $command {} 0 {} 1]
         set branches [$cmd_log\::output]
         $cmd_log\::destroy
-
         # There can be files such as "README" here that aren't branches
         set branches [grep_filter {/$} $branches]
 
-        if {[info exists cvscfg(svn_branch_filter)] && \
-            [info exists cvscfg(svn_branch_max_count)]} {
-          # Only include branches that match regexp svn_branch_filter.
-          # And on top of that, keep only the top svn_branch_max_count.
-          set branches [grep_filter $cvscfg(svn_branch_filter) $branches]
-          set branches [lrange [lsort -decreasing $branches] \
-                      0 [expr {$cvscfg(svn_branch_max_count) - 1}]]
-        }
-
         foreach branch $branches {
           gen_log:log D "$branch"
-          # There can be files such as "README" here that aren't branches
-          #if {![string match {*/} $branch]} {continue}
           set branch [string trimright $branch "/"]
           # Can't use file join or it will mess up the URL
           gen_log:log D "BRANCHES: RELPATH \"$relpath\""
           if { $relpath == {} } {
-            set path "$cvscfg(svnroot)/branches/$branch/$safe_filename"
+            set path "$cvscfg(svnroot)/$cvscfg(svn_branchdir)/$branch/$safe_filename"
           } else {
-            set path "$cvscfg(svnroot)/branches/$branch/$relpath/$safe_filename"
+            set path "$cvscfg(svnroot)/$cvscfg(svn_branchdir)/$branch/$relpath/$safe_filename"
           }
+          # Do stop-on-copy to find the base of the branch
           set command "svn log --stop-on-copy $path"
           set cmd_log [exec::new $command {} 0 {} 1]
           set log_output [$cmd_log\::output]
@@ -1465,13 +1576,35 @@ namespace eval ::svn_branchlog {
           }
           set loglines [split $log_output "\n"]
           set rb [parse_svnlog $loglines $branch]
-          #puts "parse_svnlog returned base revision $rb for $branch"
           # See if the current revision is on this branch
           set curr 0
           set brevs $branchrevs($branch)
           set tip [lindex $brevs 0]
           set revpath($tip) $path
           set revkind($tip) "revision"
+          foreach r $branchrevs($branch) {
+            if {$cvsglb(svn_mergeinfo_works) && $show_merges} {
+              set propcmd "svn propget svn:mergeinfo -$r $path"
+              # proc new {command {viewer {}} {show_stderr {1}} {filter {}} {errok {0}} }
+              set cmd_prop [exec::new $propcmd {} 0 {} 1]
+              set prop_output [$cmd_prop\::output]
+              $cmd_prop\::destroy
+              # It will probably be empty the majority of the time
+              # so don't waste time on empty output
+              if {$prop_output eq ""} {continue}
+              set splits [split $prop_output "\n"]
+              # FIXME: what if more than one merge comes into the same rev?
+              foreach line $splits {
+                if {$line eq ""} {continue}
+                set spl [split $line ":"]
+                set fromrevs [lindex $spl 1]
+                gen_log:log D "  to $r  fromrevs $fromrevs"
+                regsub {^.*-} $fromrevs {} lastfromrev 
+                set revmergefrom($r) "r$lastfromrev"
+                gen_log:log D "  revmergefrom($r) $revmergefrom($r)"
+              }
+            }
+          }
           set brevs [lreplace $brevs 0 0]
           if {$tip == $revnum_current} {
             # If current is at end of the branch do this.
@@ -1487,7 +1620,6 @@ namespace eval ::svn_branchlog {
             set revkind($r) "revision"
             set revpath($r) $path
           }
-          #set branchrevs($rb) [lrange $branchrevs($branch) 0 end-1]
           set branchrevs($rb) $branchrevs($branch)
           set revkind($rb) "branch"
           set revname($rb) $branch
@@ -1505,18 +1637,16 @@ namespace eval ::svn_branchlog {
           set loglines [split $log_output "\n"]
           parse_q $loglines $branch
 
-          # If current is HEAD of branch, move the branchpoint
-          # back one, before You are Here
+          # If current is HEAD of branch, the count is one too high because of the
+          # You Are Here box, so the branchpoint would be too low
           set idx [llength $branchrevs($branch)]
           if {$curr} {
-            #puts "Currently at Top: decrementing branchpoint"
-            gen_log:log E "Currently at Top: decrementing branchpoint"
+            gen_log:log E "Currently at Top"
             incr idx -1
           }
           set bp [lindex $allrevs($branch) $idx]
           if {$bp == ""} {
-            #puts "Allrevs same as branchrevs: decrementing branchpoint"
-            gen_log:log E "Allrevs same as branchrevs: decrementing branchpoint"
+            gen_log:log E "allrevs same as branchrevs: decrementing branchpoint"
             set bp [lindex $branchrevs($branch) end]
             set bpn [string trimleft $bp "r"]
             incr bpn -1
@@ -1525,8 +1655,9 @@ namespace eval ::svn_branchlog {
           lappend revbranches($bp) $rb
         }
         # Tags
+        # Get a list of the tags from the repository
         if {$show_tags} {
-          set command "svn list $cvscfg(svnroot)/tags"
+          set command "svn list $cvscfg(svnroot)/$cvscfg(svn_tagdir)"
           set cmd_log [exec::new $command {} 0 {} 1]
           set tags [$cmd_log\::output]
           $cmd_log\::destroy
@@ -1560,15 +1691,16 @@ namespace eval ::svn_branchlog {
             # Can't use file join or it will mess up the URL
             gen_log:log D "TAGS: RELPATH \"$relpath\""
             if { $relpath == {} } {
-              set path "$cvscfg(svnroot)/tags/$tag/$safe_filename"
+              set path "$cvscfg(svnroot)/$cvscfg(svn_tagdir)/$tag/$safe_filename"
             } else {
-              set path "$cvscfg(svnroot)/tags/$tag/$relpath/$safe_filename"
+              set path "$cvscfg(svnroot)/$cvscfg(svn_tagdir)/$tag/$relpath/$safe_filename"
             }
+            # Do log with stop-on-copy to find the actual revision that was tagged.
+            # The tag itself created a rev which may be much higher.
             set command "svn log --stop-on-copy $path"
             set cmd_log [exec::new $command {} 0 {} 1]
             set log_output [$cmd_log\::output]
             $cmd_log\::destroy
-            #update idletasks
             if {$log_output == ""} {
               continue
             }
@@ -1583,7 +1715,10 @@ namespace eval ::svn_branchlog {
             set revname($rb) "$tag"
             set revpath($rb) $path
   
-            set command "svn log -q $path"
+            # Now do log -q to find the previous rev, which is down
+            # the list.  For tags, it's only one down, so we can limit
+            # the log to 2.  It only speeds it up a little though.
+            set command "svn log -q -l 2 $path"
             set cmd_log [exec::new $command {} 0 {} 1]
             set log_output [$cmd_log\::output]
             $cmd_log\::destroy
@@ -1595,6 +1730,7 @@ namespace eval ::svn_branchlog {
             parse_q $loglines $tag
             set bp [lindex $allrevs($tag) [llength $branchrevs($tag)]]
             lappend revtags($bp) $tag
+            gen_log:log D "   revtags($bp) $revtags($bp)"
             update idletasks
           }
         }
@@ -1609,6 +1745,7 @@ namespace eval ::svn_branchlog {
         return
       }
 
+      # Parses a --stop-on-copy log, getting information for each revision
       proc parse_svnlog {lines r} {
         variable revwho
         variable revdate
@@ -1621,11 +1758,12 @@ namespace eval ::svn_branchlog {
         set i 0
         set l [llength $lines]
         while {$i < $l} {
+	  if { $i > 0 } { incr i -1 }
+	  set last [lindex $lines $i]
+	  incr i 1
           set line [lindex $lines $i]
           gen_log:log D "$i of $l:  $line"
-          if [regexp {^--*$} $line] {
-            # Next line is new revision
-            incr i
+          if { [ regexp {^[-]+$} $last ] && [ regexp {^r[0-9]+ \| .*line[s]?$} $line] } {
             if {[expr {$l - $i}] <= 1} {break}
             set line [lindex $lines $i]
             set splitline [split $line "|"]
@@ -1658,6 +1796,7 @@ namespace eval ::svn_branchlog {
         return $revnum
       }
 
+      # Parses a summary (-q) log to find what revisions are on it
       proc parse_q {lines r} {
         variable allrevs
 
@@ -1689,6 +1828,7 @@ namespace eval ::svn_branchlog {
         variable revbtags
         variable branchrevs
         variable revbranches
+        variable revmergefrom
         variable logstate
         variable revnum
         variable rootbranch
@@ -1721,6 +1861,10 @@ namespace eval ::svn_branchlog {
            foreach a [lsort -dictionary [array names revtags]] {
            gen_log:log D "revtags($a) $revtags($a)"
         }
+        gen_log:log D ""
+           foreach a [lsort -dictionary [array names revmergefrom]] {
+           gen_log:log D "revmergefrom($a) $revmergefrom($a)"
+        }
         # We only needed these to place the you-are-here box.
         catch {unset rootbranch revbranch}
         $ln\::DrawTree now
